<?php
// Not allowed by directly accessing.
if ( ! defined( 'ABSPATH' ) ) {
	die( 0 );
}

if ( ! class_exists( 'LoftLoader_Pro_Frontend_Manager' ) ) {
	/**
	 * @since version 1.0
	 */
	class LoftLoader_Pro_Frontend_Manager {
		/**
		* Boolean to tell if enable session
		*/
		protected $enabled_session = false;
		/**
		* Boolean to tell if disable page scrolling funtion during loading
		*/
		protected $disable_page_scroll = false;
		/**
		* Boolean to tell if currently in mobile mode
		*/
		protected $is_mobile = false;
		/**
		* If head loaded
		*/
		protected $site_header_loaded = false;
		/**
		* If footer loaded
		*/
		protected $site_footer_loaded = false;
		/**
		* Contruct function
		*/
		public function __construct() {
			$this->includes();

			add_action( 'wp', 								array( $this, 'init' ) );
			add_action( 'template_redirect',				array( $this, 'start_cache' ) );
			add_filter( 'loftloader_pro_loader_enabled', 	array( $this, 'loader_enabled' ) );
			add_filter( 'loftloader_pro_loader_attributes', array( $this, 'add_loader_wrapper_attributes' ) );
			add_filter( 'loftloader_pro_loader_enabled', 	array( $this, 'detect_url_switcher' ), 999 );
		}
		/**
		* Test if page loader if switch off via url by add pageloader=false
		* @param boolean
		* @return boolean
		*/
		public function detect_url_switcher( $status ) {
			if ( isset( $_GET['pageloader'] ) && ( 'false' === sanitize_text_field( wp_unslash( $_GET['pageloader'] ) ) ) ) {
				return false;
			} else {
				return $status;
			}
		}
		/**
		* Start cache for outputing
		*/
		public function start_cache() {
			// Only for front view
			if ( ! is_admin() && ! wp_doing_ajax() ) {
				// Start cache the output with callback function
				ob_start();
				add_action( 'shutdown', array( $this, 'modify_html' ) , -1 );
			}
		}
		/**
		* Will be called when flush cache
		*
		* @param string cached string
		* @return string modified cached string
		*/
		public function modify_html() {
			$html = ob_get_clean();
			if ( $this->site_header_loaded && $this->site_footer_loaded ) {
				echo apply_filters( 'loftloader_pro_modify_html', $html );
			} else {
				echo $html;
			}
		}
		/**
		* Include files required
		*/
		protected function includes() {
			$inc = LOFTLOADERPRO_INC . 'frontend/';
			require_once $inc . 'class-custom-styles.php';
			require_once $inc . 'class-loader-html.php';
			require_once LOFTLOADERPRO_INC . 'vender/Mobile_Detect.php';
		}
		/**
		* Initialize loader
		*/
		public function init() {
			// Initialize any page settings if needed
			do_action( 'loftloader_pro_settings' );
			$detect = new Mobile_Detect();
			$this->is_mobile = $detect->isMobile() || $detect->isTablet();

			// Only run if loader enabled and on frontend
			if ( ! is_admin() ) {
				if ( apply_filters( 'loftloader_pro_loader_enabled', false ) ) {
					add_action( 'wp_head', 				array( $this, 'set_site_header_loaded' ), 99 );
					add_action( 'wp_footer', 			array( $this, 'set_site_footer_loaded' ), 99 );
					add_action( 'template_redirect', 	array( $this, 'init_front' ), 100000 );
					add_action( 'wp_head', 				array( $this, 'enqueue_styles' ), 0 );
					add_action( 'wp_enqueue_scripts', 	array( $this, 'enqueue_scripts' ) );
					add_filter( 'body_class',			array( $this, 'set_body_class' ) );
					add_filter( 'loftloader_pro_modify_html', array( $this, 'add_inline_to_head' ), 5 );
					add_filter( 'loftloader_pro_enabled_session', array( $this, 'is_session_enabled' ) );
				} else {
					add_action( 'wp_head', array( $this, 'clear_localstorage' ), 99 );
				}
			}
		}
		/**
		* Initialize loader front
		*/
		public function init_front() {
			// Set the page scroll while loading setting
			$this->disable_page_scroll = llp_module_enabled( 'loftloader_pro_disable_page_scrolling' ) && ! is_customize_preview();
			// Roll the front related functions
			do_action( 'loftloader_pro_init_front', $this->is_mobile );
		}
		/**
		* Check if enable the option disbale page scroll while loading, if so add class to <body>
		*
		* @param array class list
		* @return array refined class list
		*/
		public function set_body_class( $class ) {
			array_push( $class, 'loftloader-pro-enabled' );
			// If page scroll while loading is disabled, add the class to <body>
			if ( $this->disable_page_scroll ) {
				array_push( $class, 'loftloader-disable-scrolling' );
			}
			return $class;
		}
		public function enqueue_styles() {
			$enable_message_google_font = llp_module_enabled( 'loftloader_pro_message_enable_google_font' );
			$enable_progress_google_font = llp_module_enabled( 'loftloader_pro_progress_number_enable_google_font' );
			if ( $enable_progress_google_font || $enable_message_google_font ) {
				// Enqueue the google font if needed
				$google_font 	= array();
				$number_font 	= llp_get_loader_setting( 'loftloader_pro_progress_number_font_family' );
				$message_font 	= llp_get_loader_setting( 'loftloader_pro_message_font_family' );
				if ( $enable_progress_google_font && ! empty( $number_font ) ) {
					$google_font[] = str_replace( ' ', '+', $number_font ) . ':100,200,300,400,500,600,700,800';
				}
				if ( $enable_message_google_font && ! empty( $message_font ) ) {
					$google_font[] = str_replace( ' ', '+', $message_font ) . ':100,200,300,400,500,600,700,800';
				}
				if ( ! empty( $google_font ) ) {
					$fonts_url = add_query_arg( array(
						'family' => urlencode( implode( '|', array_unique( $google_font ) ) ),
					), 'https://fonts.googleapis.com/css' );

					wp_enqueue_style( 'loftloader-google-font', $fonts_url, array(), LOFTLOADERPRO_ASSET_VERSION );
				}
			}
			// Enqueue the main loader style file
			wp_enqueue_style( 'loftloader-style', LOFTLOADERPRO_ASSETS_URI . 'css/loftloader.min.css', array(), LOFTLOADERPRO_ASSET_VERSION );
		}
		/**
		* Cache script
		*/
		public function inline_scripts() {
			if ( ! is_customize_preview() ) {
				$assets = LOFTLOADERPRO_ROOT . 'assets/js/';
				ob_start();
				// Cache related script
				if ( defined( 'WP_CACHE' ) && WP_CACHE ) {
					require_once $assets . 'loftloader-cache.php';
				}
				require_once $assets . 'loftloader-global.php';
				return ob_get_clean();
			}
			return '';
		}
		/**
		 * @description enqueue the scripts and styles for front end
		 */
		public function enqueue_scripts() {
			$asset_uri = LOFTLOADERPRO_ASSETS_URI;
			$asset_version = LOFTLOADERPRO_ASSET_VERSION;
			$elements = llp_get_loader_setting( 'loftloader_pro_detect_elements' );
			// Register the require jquery plugin
			wp_register_script( 'jquery-waitformedias', $asset_uri . 'js/jquery.waitformedias.min.js', array( 'jquery' ), $asset_version, true );
			wp_localize_script(
				'jquery-waitformedias',
				'loftloaderDetectElement',
				esc_js( $elements )
			);
			// Enqueue the main loader javascript
			wp_register_script( 'loftloader-front-main', $asset_uri . 'js/loftloader.min.js', array( 'jquery-waitformedias' ), $asset_version, true );
			$list = llp_get_random_message_list();
			$js_vars = array( 'leavingProgressMax' => '0.6', 'insiteTransitionShowAll' => llp_module_enabled( 'loftloader_pro_insite_transition_show_all' ) );
			if ( llp_module_enabled( 'loftloader_pro_enable_random_message_text' ) && llp_module_enabled( 'loftloader_pro_render_random_message_by_js' ) && ! empty( $list ) ) {
				$js_vars[ 'randomMessage' ] = $list;
			}
			wp_localize_script( 'loftloader-front-main', 'loftloaderPro', $js_vars );
			wp_enqueue_script( 'loftloader-front-main' );
		}
		/**
		* Add inline styles and scripts
		*/
		public function add_inline_to_head( $html ) {
			if ( ! empty( $html ) ) {
				$regexp_html ='/(<head[^>]*>)/i';
				$split = preg_split( $regexp_html, $html, 2, PREG_SPLIT_DELIM_CAPTURE );
				if ( is_array( $split ) && ( 3 === count( $split ) ) ) {
					// Not show the loader if javascript is disabled
					$inline = '<noscript><style>#loftloader-wrapper { display: none !important; }</style></noscript>';
					// For cache related style
					$inline .= '<style> html.loftloader-pro-hide #loftloader-wrapper { display: none !important; } </style>';
					// Add page scroll while loading disabled related inline styles
					if ( $this->disable_page_scroll ){
						$inline .= '<style id="loftloader-pro-disable-scrolling"> body.loftloader-disable-scrolling { overflow: hidden !important; max-height: 100vh !important; height: 100%; position: fixed !important; width: 100% } </style>';
						$inline .= '<style id="loftloader-pro-always-show-scrollbar"> html { overflow-y: scroll !important; } </style>';
					}
					// Add insite transition related inline styles
					if ( $this->test_insite_transition() ) {
						$styles = $this->get_html_styles();
						if ( ! empty( $styles ) ) {
							$inline .= llp_generate_style(
								'loftloader-page-smooth-transition-bg',
								sprintf(
									'html.loftloader-smooth-transition { %s }',
									wp_kses( $styles, array( "\'", '\"' ) )
								)
							);
							$inline .= '<script type="text/javascript"> document.documentElement.classList.add( "loftloader-smooth-transition" ); </script>';
						}
					}
					$inline .= $this->inline_scripts();
					$split[1] .= $inline;
					$html = implode( '', $split );
				}
			}
			return $html;
		}
		/**
		* Test whether show loftloader
		* @return boolean return true if loftloader enabled and display on current page, otherwise false
		*/
		public function loader_enabled() {
			if ( ( 'on' === esc_attr( llp_get_loader_setting( 'loftloader_pro_main_switch') ) ) && $this->device_test() ) {
				$this->check_cookies();
				$queried = get_queried_object();
				$range = llp_get_loader_setting( 'loftloader_pro_show_range' );
				// Home only + once per session
				if ( ( $range === 'homepage-once' ) && is_front_page() && $this->once_test( 'homepage') ) {
	    			$_SESSION['loftloader-pro-homepage-visited'] = true;
					return true;
				} else if ( ( $range === 'once' ) && $this->once_test() ) { // Once per session
					$_SESSION['loftloader-pro-visited'] = true;
					return true;
				} else if ( $range === 'post_types' ) { // Sitewide - selected post types
					$types = ( array )llp_get_loader_setting( 'loftloader_pro_post_types' );
					return empty( $types ) || !is_singular( $types );
				} else if ( $range === 'selected_post_types' ) { // Selected post types
					$types = (array) llp_get_loader_setting( 'loftloader_pro_selected_post_types' );
					return ! empty( $types ) && is_singular( $types );
				} else if ( ( $range === 'sitewide' )
					|| ( ( $range === 'all' ) && ( (  $this->is_singles( $queried ) && ( $queried->post_type === 'page' ) ) || is_home() || $this->is_woocommerce_shop() ) )
					|| ( ( $range === 'homepage' ) && is_front_page() ) ) { // Sitewide || all pages || homepage only

					return true;
				} else if ( $range === 'handpick' ) { // Handpick
					$pages = llp_get_loader_setting( 'loftloader_pro_hand_pick_pages' );
					// Support woocommerce shop page
					return ( $this->is_singles( $queried ) && in_array( $queried->ID, $pages ) )
						|| ( $this->is_woocommerce_shop() && in_array( wc_get_page_id( 'shop' ), $pages ) );
				}
			}
		}
		/**
		* Add insite transition attributes to loader wrapper
		*
		* @param string original attributes
		* @return string modified attributes
		*/
		public function add_loader_wrapper_attributes( $attrs ) {
			if ( $this->test_insite_transition() ) {
				$exclude = llp_get_loader_setting( 'loftloader_pro_exclude_from_page_transition' );
				$attrs 	.= ' data-insite-transition="on"';
				$attrs 	.= ' data-site-url="' . esc_url( get_site_url() ) . '"';

				if ( ! empty( $exclude ) ) {
					$attrs .= sprintf( ' data-insite-transition-excluded="%s"', esc_attr( esc_js( $exclude ) ) );
				}
			}
			return $attrs;
		}
		/**
		* Set cookies if once per session enabled
		*/
		protected function check_cookies() {
			$range = esc_attr( llp_get_loader_setting( 'loftloader_pro_show_range', true ) );
			$onces = array( 'homepage-once', 'once' );
			if ( in_array( $range, $onces ) && ! is_customize_preview() && ! headers_sent() ) {
				if ( ( 'homepage-once' === $range ) && is_front_page() ) {
					llp_set_cookie( 'loftloader_pro_homepage_once_per_session_visited', 'on' );
					$this->enabled_session = true;
				} else if ( 'once' === $range ){
					llp_set_cookie( 'loftloader_pro_once_per_session_visited', 'on' );
					$this->enabled_session = true;
				}
				llp_set_cookie( 'wordpress-loftloader_pro_session', 'on' );
				nocache_headers();
			}
		}
		/**
		* Test current settings for once per session
		* @return boolean, return false if visited, otherwise true
		*/
		protected function once_test( $id = '' ) {
			if ( $this->enabled_session ) {
				$cookie_id = ( 'homepage' === $id ) ? 'loftloader_pro_homepage_once_per_session_visited' : 'loftloader_pro_once_per_session_visited';
				return empty( $_COOKIE[ $cookie_id ] ) || ( 'on' !== sanitize_text_field( wp_unslash( $_COOKIE[ $cookie_id] ) ) );
			} else {
				return true;
			}
		}
		/**
		* Test current device and check whether to show loftloader
		* @return boolean
		*/
		protected function device_test() {
			// Always return true when in customizer page.
			if ( is_customize_preview() ) {
				return true;
			}

			$device = llp_get_loader_setting( 'loftloader_pro_device' );
			switch ( $device ) {
				case 'all':
					return true;
				case 'notmobile':
					return ! $this->is_mobile;
				case 'mobileonly':
					return $this->is_mobile;
				defaults:
					return false;
			}
		}
		/**
		* Check if enable insite page transition
		* @return boolean
		*/
		protected function test_insite_transition() {
			return llp_module_enabled( 'loftloader_pro_insite_transition' )
				&& ! in_array( llp_get_loader_setting( 'loftloader_pro_show_range'), array( 'once', 'homepage-once' ) );

		}
		/**
		* Get initial styles for <html>
		*/
		protected function get_html_styles() {
			$bg_type 				= llp_get_loader_setting( 'loftloader_bgfilltype' );
			$bg_color 				= llp_get_loader_setting( 'loftloader_pro_bg_color' );
			$gradient_angel 		= llp_get_loader_setting( 'loftloader_pro_bg_gradient_angel' );
			$gradient_start_color 	= llp_get_loader_setting( 'loftloader_pro_bg_gradient_start_color' );
			$gradient_end_color 	= llp_get_loader_setting( 'loftloader_pro_bg_gradient_end_color' );
			$image_url 				= llp_get_loader_setting( 'loftloader_pro_bg_image' );
			$styles 				= sprintf( 'background-color: %s;', llp_get_loader_setting( 'loftloader_pro_bg_color' ) );

			switch ( $bg_type ) {
				case 'solid':
					if ( llp_module_enabled( 'loftloader_pro_bg_gradient' ) ) {
						$gradient 	= sprintf( '(%sdeg, %s, %s);', $gradient_angel, $gradient_start_color, $gradient_end_color );
						$prefix 	= array( '-webkit-linear-gradient', '-o-linear-gradient', '-moz-linear-gradient', 'linear-gradient' );
						foreach( $prefix as $p ) {
							$styles	.= sprintf( ' background-image: %s%s', $p, $gradient );
						}
					}
					if ( ! empty( $bg_color ) ) {
						$styles .= sprintf( ' background-color: %s', $bg_color );
					}
					break;
				case 'image':
					if ( ! empty( $image_url ) ) {
						$styles .= sprintf( ' background-image: url(%s);', esc_url( $image_url ) );
					}
					break;
			}
			return $styles;
		}
		/**
		* Test is woocommerce shop page
		*/
		protected function is_woocommerce_shop() {
			if ( function_exists( 'is_shop' ) ) {
				$page_id = wc_get_page_id( 'shop' );
				return !empty( $page_id ) && ( $page_id !== -1 ) && is_shop();
			}

			return false;
		}
		/**
		* Test if current request is for single page not archive page
		* @param mix
		* @return boolean
		*/
		protected function is_singles( $query ) {
			return is_object( $query ) && ( 'WP_Post' === get_class( $query ) );
		}
		/**
		* Set site header loaded to true
		*/
		public function set_site_header_loaded() {
			$this->site_header_loaded = true;
		}
		/**
		* Set site footer loaded to true
		*/
		public function set_site_footer_loaded() {
			$this->site_footer_loaded = true;
		}
		/**
		* Clear browser local storage
		*/
		public function clear_localstorage() {
			if ( $this->test_insite_transition() ) : ?>
			<script type="text/javascript">
				if ( window.sessionStorage.getItem( 'loftloader-pro-smooth-transition' ) && ( 'on' === window.sessionStorage.getItem( 'loftloader-pro-smooth-transition' ) ) ) {
					window.sessionStorage.setItem( 'loftloader-pro-smooth-transition', 0 );
				}
			</script> <?php
			endif;
		}
		/**
		* If in once per session
		*/
		public function is_session_enabled( $enabled ) {
			return $this->enabled_session;
		}
	}
	new LoftLoader_Pro_Frontend_Manager();
}
